/*
 * Copyright (c) 2019 Samsung Electronics Co., Ltd All Rights Reserved
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
#include <gmock/gmock.h>
#include <gtest/gtest.h>
#include <unistd.h>
#include <stdio.h>

#include <appsvc.h>

#include "unittest.h"
#include "alarm.h"
#include "mock/gio_fake.h"
#include "mock/alarm_dbus.h"
#include "mock/aul_fake.h"
#include "mock/notification_fake.h"

void __g_dbus_node_info_unref_fake(GDBusNodeInfo *info)
{
  free(info->interfaces);
  free(info);
  info = NULL;
  return;
}

GDBusConnection* __g_bus_get_sync_fake(GBusType type,
    GCancellable *cancellable, GError **error)
{
  GDBusConnection *con = (GDBusConnection*)g_object_new(G_TYPE_OBJECT, NULL);

  return con;
}

GDBusProxy* __g_dbus_proxy_new_sync_fake(GDBusConnection *con,
    GDBusProxyFlags flag, GDBusInterfaceInfo *info, const gchar *name,
    const gchar *path, const gchar *interface, GCancellable *cancellable,
    GError **error)
{
  GDBusProxy *proxy = (GDBusProxy*)g_object_new(G_TYPE_OBJECT, NULL);

  return proxy;
}

GDBusNodeInfo* __g_dbus_node_info_new_for_xml_fake(const gchar *xml, GError **error)
{
  GDBusNodeInfo *info = (GDBusNodeInfo*)malloc(sizeof(GDBusNodeInfo));

  info->ref_count = 10;
  info->path = NULL;
  info->interfaces = (GDBusInterfaceInfo**)malloc(sizeof(GDBusInterfaceInfo));
  info->nodes = NULL;
  info->annotations = NULL;

  return info;
}

guint __g_dbus_connection_register_object_fake(GDBusConnection *con,
    const gchar *path, GDBusInterfaceInfo *info,
    const GDBusInterfaceVTable *table, gpointer user_data,
    GDestroyNotify user_data_free_func, GError **error)
{
  return 1;
}

guint __g_bus_own_name_on_connection_fake(GDBusConnection *con, const gchar *name,
    GBusNameOwnerFlags flag, GBusNameAcquiredCallback acquired_cb,
    GBusNameLostCallback lost_cb, gpointer user_data,
    GDestroyNotify user_data_free_func)
{
  return 1;
}

gboolean __alarm_manager_call_alarm_get_info_sync_fake(AlarmManager *alarm,
    gint alarm_id, gint *start_year, gint *start_month, gint *start_day, gint *start_hour,
    gint *start_min, gint *start_sec, gint *end_year, gint *end_month, gint *end_day,
    gint *mode_day_of_week, gint *mode_repeat, gint *alarm_type,
    gint *reserved_info, gint *return_code, GCancellable *cancel, GError **error)
{
  return true;
}

//gboolean __alarm_manager_call_alarm_create_sync_fake(AlarmManager *alarm,
//    const gchar *arg_app_service_name, const gchar *arg_app_service_name_mod,
//    gint start_year, gint start_month,
//    gint start_day, gint start_hour, gint start_min, gint start_sec,
//    gint msec, gint end_year, gint end_month, gint end_day,
//    gint mode_day_of_week, gint mode_repeat, gint alarm_type,
//    gint reserved_info, const gchar *arg_alarm_info_reserved_service_name,
//    const gchar *arg_alarm_info_reserved_service_name_mod, gint *alarm_id,
//    gint *return_code, GCancellable *cancel, GError **error)
//{
//  return true;
//}

gboolean __alarm_manager_call_alarm_create_appsvc_sync_fake(AlarmManager *alarm,
    gint start_year, gint start_month, gint start_day, gint start_hour,
    gint start_min, gint start_sec, gint end_year, gint end_month, gint end_day,
    gint mode_day_of_week, gint mode_ginterval, gint mode_repeat, gint alarm_type,
    gint reserved_info, const char *bundle, gint *alarm_id, gint *return_code,
    GCancellable *cancel, GError **error)
{
  return true;
}

gboolean __alarm_manager_call_alarm_create_noti_sync_fake(AlarmManager *alarm,
    gint start_year, gint start_month, gint start_day, gint start_hour,
    gint start_min, gint start_sec, gint end_year, gint end_month, gint end_day,
    gint mode_day_of_week, gint mode_ginterval, gint mode_repeat, gint alarm_type,
    gint reserved_info, const char *bundle, gint *alarm_id, gint *return_code,
    GCancellable *cancel, GError **error)
{
  return true;
}

gboolean __alarm_manager_call_alarm_get_appsvc_info_sync_fake(AlarmManager *alarm,
    gint alarm_id, gchar **b, gint *return_code, GCancellable *cancel,
    GError **error)
{
  int datalen;
  bundle_raw *raw = NULL;
  bundle *bundle = bundle_create();
	bundle_add_str(bundle, "key", "val");

  bundle_encode(bundle, &raw, &datalen);

  *b = (gchar *)raw;
  *return_code = 0;

  bundle_free(bundle);

  return true;
}

gboolean __alarm_manager_call_alarm_get_noti_info_sync_fake(AlarmManager *alarm,
    gint alarm_id, gchar **b, gint *return_code, GCancellable *cancel,
    GError **error)
{
  GVariant *noti_gv = NULL;
  char *noti_data;
  guchar *data;
  int datalen = 0;
  int noti_ret;
  notification_h noti;

  noti = notification_create(NOTIFICATION_TYPE_NOTI);
  if (!noti)
    return false;

  noti_ret = notification_set_text(noti, NOTIFICATION_TEXT_TYPE_TITLE,
      "Test Title", "TITLE", NOTIFICATION_VARIABLE_TYPE_NONE);
  if (noti_ret != NOTIFICATION_ERROR_NONE)
    return false;

  noti_gv = notification_ipc_make_gvariant_from_noti(noti, true);
  if (!noti_gv)
    return false;

  datalen = g_variant_get_size(noti_gv);
  if (datalen < 0)
    return false;

  data = (guchar *)malloc(datalen);
  if (!data)
    return false;

  g_variant_store(noti_gv, data);
  noti_data = g_base64_encode((guchar *)data, datalen);

  *b = noti_data;

  return true;
}

gboolean __alarm_manager_call_alarm_set_rtc_time_sync_fake(AlarmManager *alarm,
    gint year, gint month, gint day, gint hour, gint min, gint sec, gint *return_code, GCancellable *cancel, GError **error)
{
  return true;
}

gboolean __alarm_manager_call_alarm_delete_sync_fake(AlarmManager *alarm,
    gint alarm_id, gint *return_code, GCancellable *cancel,
    GError **error)
{
  return true;
}

gboolean __alarm_manager_call_alarm_delete_all_sync_fake(AlarmManager *alarm,
    gint *return_code, GCancellable *cancel, GError **error)
{
  return true;
}

gboolean __alarm_manager_call_alarm_get_number_of_ids_sync_fake(AlarmManager *alarm,
    gint *num, gint *return_code, GCancellable *cancel, GError **error)
{
  return true;
}

gboolean __alarm_manager_call_alarm_get_list_of_ids_sync_fake(AlarmManager *alarm,
    gint id, GVariant **alarm_array, gint *num_of_ids, gint *return_code,
    GCancellable *cancel, GError **error)
{
  int test[2] = {1, 1};
  GVariantBuilder *builder;

	builder = g_variant_builder_new(G_VARIANT_TYPE("ai"));
  g_variant_builder_add (builder, "i", test[0]);
  g_variant_builder_add (builder, "i", test[1]);

  *alarm_array = g_variant_new("ai", builder);

  return true;
}

gboolean __alarm_manager_call_alarm_get_next_duetime_sync_fake(AlarmManager *alarm,
    gint alarm_id, gint *duetime, gint *return_code, GCancellable *cancel,
    GError **error)
{
  return true;
}

gboolean __alarm_manager_call_alarm_create_periodic_sync_fake(AlarmManager *alarm,
    const gchar* service_name, const gchar *service_name_mod, gint interval,
    gint is_ref, gint method, gint *alarm_id, gint *return_code,
    GCancellable *cancel, GError **error)
{
  return true;
}

gboolean __alarm_manager_call_alarm_set_time_sync_fake(AlarmManager *alarm,
    gint new_time, gint *return_code, GCancellable *cancel, GError **error)
{
  return true;
}

void __alarm_manager_call_alarm_set_time_fake(AlarmManager *alarm,
    gint new_time, GCancellable *cancel, GAsyncReadyCallback cb, gpointer user_data)
{
  return;
}

gboolean __alarm_manager_call_alarm_set_timezone_sync_fake(AlarmManager *alarm,
    const gchar* tzpath, gint *return_code, GCancellable *cancel, GError **error)
{
  return true;
}

gboolean __alarm_manager_call_alarm_set_time_with_propagation_delay_sync_fake(AlarmManager *alarm,
    guint new_sec, guint new_nsec, guint req_sec, guint req_nsec,
    gint *return_code, GCancellable *cancel, GError **error)
{
  return true;
}

void __alarm_manager_call_alarm_set_time_with_propagation_delay_fake(AlarmManager *alarm,
    guint new_sec, guint new_nsec, guint req_sec, guint req_nsec,
    GCancellable *cancel, GAsyncReadyCallback, gpointer user_data)
{
  return;
}

gboolean __alarm_manager_call_alarm_update_sync_fake(AlarmManager *alarm,
    gint id, gint start_year, gint start_month, gint start_day, gint start_hour,
    gint start_min, gint start_sec, gint end_year, gint end_month, gint end_day,
    gint mode_ginterval, gint mode_repeat, gint alarm_type, gint reserved_info,
    gint update_flag, gint *return_code, GCancellable *cancel, GError **error)
{
  return true;
}

gboolean __alarm_manager_call_alarm_set_global_sync_fake(AlarmManager *alarm,
    gint id, gboolean global, gint *return_code, GCancellable *cancel, GError **error)
{
  return true;
}

gboolean __alarm_manager_call_alarm_get_global_sync_fake(AlarmManager *alarm,
    gint id, gboolean *global, gint *return_code, GCancellable *cancel, GError **error)
{
  return true;
}

int __aul_app_get_appid_bypid_fake(int pid, char *appid, int length)
{
  sprintf(appid, "%s", "test");

  return 0;
}

GVariant* __notification_ipc_make_gvariant_from_noti_fake(notification_h,
    bool translate)
{
  GVariant *noti_gv = NULL;

  noti_gv = g_variant_new_string("test");

  return noti_gv;
}

class AlarmLibTest : public ::testing::Test {
  protected:
    void SetUp() override {
      alarm = alarmmgr_create_alarm();

      g_bus_get_sync_fake.custom_fake = __g_bus_get_sync_fake;
      g_dbus_proxy_new_sync_fake.custom_fake = __g_dbus_proxy_new_sync_fake;
      g_dbus_node_info_unref_fake.custom_fake = __g_dbus_node_info_unref_fake;

    }

    void TearDown() override {
      alarmmgr_free_alarm(alarm);
    }

    alarm_entry_t *alarm;
};

TEST_F(AlarmLibTest, alarmmgr_init_p)
{
  int ret;
  char testapp[256] = "org.tizen.alarmmgrtestapp";

  g_dbus_node_info_new_for_xml_fake.custom_fake = __g_dbus_node_info_new_for_xml_fake;
  g_dbus_connection_register_object_fake.custom_fake = __g_dbus_connection_register_object_fake;
  g_bus_own_name_on_connection_fake.custom_fake = __g_bus_own_name_on_connection_fake;

  ret = alarmmgr_init(testapp);
  EXPECT_EQ(ALARMMGR_RESULT_SUCCESS, ret);
}

TEST_F(AlarmLibTest, alarmmgr_set_get_time_p)
{
  int ret;
  alarm_date_t date;
  alarm_date_t get_date;

  date.year = 2020;
  date.month = 1;
  date.day = 1;
  date.hour = 1;
  date.min = 1;
  date.sec = 1;

  ret = alarmmgr_set_time(alarm, date);
  EXPECT_EQ(ALARMMGR_RESULT_SUCCESS, ret);

  ret = alarmmgr_get_time(alarm, &get_date);
  EXPECT_EQ(ALARMMGR_RESULT_SUCCESS, ret);

  EXPECT_EQ(date.year, get_date.year);
}

TEST_F(AlarmLibTest, alarmmgr_set_get_repeat_mode_p)
{
  int ret;
  alarm_repeat_mode_t get_mode;
  int interval;

  ret = alarmmgr_set_repeat_mode(alarm, ALARM_REPEAT_MODE_REPEAT, 1000);
  EXPECT_EQ(ALARMMGR_RESULT_SUCCESS, ret);

  ret = alarmmgr_get_repeat_mode(alarm, &get_mode, &interval);
  EXPECT_EQ(ALARMMGR_RESULT_SUCCESS, ret);

  EXPECT_EQ(ALARM_REPEAT_MODE_REPEAT, get_mode);
}

TEST_F(AlarmLibTest, alarmmgr_set_get_type_p)
{
  int ret;
  int get_type;

  ret = alarmmgr_set_type(alarm, ALARM_TYPE_VOLATILE);
  EXPECT_EQ(ALARMMGR_RESULT_SUCCESS, ret);

  ret = alarmmgr_get_type(alarm, &get_type);
  EXPECT_EQ(ALARMMGR_RESULT_SUCCESS, ret);

  EXPECT_EQ(ALARM_TYPE_VOLATILE, get_type);
}

TEST_F(AlarmLibTest, alarmmgr_add_alarm_appsvc_p)
{
  int ret;
  alarm_id_t alarm_id;

  bundle *b;

  alarm_manager_call_alarm_create_appsvc_sync_fake.custom_fake = __alarm_manager_call_alarm_create_appsvc_sync_fake;

  b = bundle_create();
  ASSERT_NE(b, nullptr);

  ret = appsvc_set_pkgname(b,"org.tizen.calendar");
  EXPECT_EQ(0, ret);

  ret = appsvc_set_operation(b, APPSVC_OPERATION_DEFAULT);
  EXPECT_EQ(0, ret);

  ret = alarmmgr_add_alarm_appsvc(ALARM_TYPE_DEFAULT, 100, 0, (void *)b,
      &alarm_id);
  EXPECT_EQ(ALARMMGR_RESULT_SUCCESS, ret);

  bundle_free(b);
}

TEST_F(AlarmLibTest, alarmmgr_get_alarm_appsvc_p)
{
  int ret;
  alarm_id_t alarm_id = 10;

  bundle *get_b;

  alarm_manager_call_alarm_get_appsvc_info_sync_fake.custom_fake = __alarm_manager_call_alarm_get_appsvc_info_sync_fake;

  get_b = (bundle *)alarmmgr_get_alarm_appsvc_info(alarm_id, &ret);
  EXPECT_EQ(ALARMMGR_RESULT_SUCCESS, ret);

  bundle_free(get_b);
}


TEST_F(AlarmLibTest, alarmmgr_add_alarm_noti_p)
{
  int ret;
  alarm_id_t alarm_id;
  notification_h noti = nullptr;

  alarm_manager_call_alarm_create_noti_sync_fake.custom_fake = __alarm_manager_call_alarm_create_noti_sync_fake;
  notification_ipc_make_gvariant_from_noti_fake.custom_fake = __notification_ipc_make_gvariant_from_noti_fake;

  ret = alarmmgr_add_alarm_noti(ALARM_TYPE_DEFAULT, 100, 0, noti,
      &alarm_id);
  EXPECT_EQ(ALARMMGR_RESULT_SUCCESS, ret);
}


TEST_F(AlarmLibTest, alarmmgr_get_alarm_noti_p)
{
  int ret;
  notification_h get_noti;

  alarm_manager_call_alarm_get_noti_info_sync_fake.custom_fake = __alarm_manager_call_alarm_get_noti_info_sync_fake;

  ret = alarmmgr_get_alarm_noti_info(1, &get_noti);
  EXPECT_EQ(ALARMMGR_RESULT_SUCCESS, ret);
}

TEST_F(AlarmLibTest, alarmmgr_set_rtc_time_p)
{
  int ret;
  alarm_date_t date = {2020, 4, 5, 10, 10, 0};

  alarm_manager_call_alarm_set_rtc_time_sync_fake.custom_fake = __alarm_manager_call_alarm_set_rtc_time_sync_fake;

  ret = alarmmgr_set_rtc_time(&date);
  EXPECT_EQ(ALARMMGR_RESULT_SUCCESS, ret);
}

TEST_F(AlarmLibTest, alarmmgr_add_alarm_appsvc_with_localtime_p)
{
  int ret;
  alarm_id_t alarm_id;

  bundle *b;

  alarm_date_t test_time;

  alarm_manager_call_alarm_create_appsvc_sync_fake.custom_fake = __alarm_manager_call_alarm_create_appsvc_sync_fake;

  b = bundle_create();
  ASSERT_NE(b, nullptr);

  ret = appsvc_set_pkgname(b,"org.tizen.calendar");
  EXPECT_EQ(0, ret);

  ret = appsvc_set_operation(b, APPSVC_OPERATION_DEFAULT);
  EXPECT_EQ(0, ret);

  test_time.year = 2022;
  test_time.month = 1;
  test_time.day = 1;

  test_time.hour = 1;
  test_time.min = 1;
  test_time.sec = 5;

  ret = alarmmgr_set_time(alarm,test_time);
  EXPECT_EQ(ALARMMGR_RESULT_SUCCESS, ret);

  ret = alarmmgr_set_type(alarm, ALARM_TYPE_DEFAULT);
  EXPECT_EQ(ALARMMGR_RESULT_SUCCESS, ret);

  ret = alarmmgr_add_alarm_appsvc_with_localtime(alarm, (void *)b,
      &alarm_id);
  EXPECT_EQ(ALARMMGR_RESULT_SUCCESS, ret);

  bundle_free(b);
}

TEST_F(AlarmLibTest, alarmmgr_get_info_p)
{
  int ret;
  alarm_entry_t get_alarm;

  alarm_manager_call_alarm_get_info_sync_fake.custom_fake = __alarm_manager_call_alarm_get_info_sync_fake;

  ret = alarmmgr_get_info(1, &get_alarm);
  EXPECT_EQ(ALARMMGR_RESULT_SUCCESS, ret);
}

TEST_F(AlarmLibTest, alarmmgr_add_alarm_noti_with_localtime_p)
{
  int ret;
  alarm_id_t alarm_id;
  notification_h noti = nullptr;

  alarm_manager_call_alarm_create_noti_sync_fake.custom_fake = __alarm_manager_call_alarm_create_noti_sync_fake;

  ret = alarmmgr_add_alarm_noti_with_localtime(alarm, noti, &alarm_id);
  EXPECT_EQ(ALARMMGR_RESULT_SUCCESS, ret);
}

//TODO
/*
TEST_F(AlarmLibTest, alarmmgr_add_alarm_p)
{
  int ret;
  alarm_id_t alarm_id;
  alarm_entry_t get_alarm;

  alarm_manager_call_alarm_create_sync_fake.custom_fake =  __alarm_manager_call_alarm_create_sync_fake;

  ret = alarmmgr_add_alarm(ALARM_TYPE_VOLATILE, 100, 1000, "gtest-alarmmgr",
      &alarm_id);
  EXPECT_EQ(ALARMMGR_RESULT_SUCCESS, ret);
}

TEST_F(AlarmLibTest, alarmmgr_add_alarm_precision_p)
{
  int ret;
  alarm_id_t alarm_id;
  alarm_entry_t get_alarm;

  ret = alarmmgr_add_alarm_precision(ALARM_TYPE_VOLATILE, 100, 1000,
      "gtest-alarmmgr", &alarm_id);
  EXPECT_EQ(ALARMMGR_RESULT_SUCCESS, ret);
}

TEST(AlarmLibTest, alarmmgr_add_get_alarm_withcb_p)
{
}

TEST(AlarmLibTest, alarmmgr_add_get_alarm_withcb_precision_p)
{
 }
*/

TEST_F(AlarmLibTest, alarmmgr_remove_alarm_p)
{
  int ret;
  alarm_manager_call_alarm_delete_sync_fake.custom_fake =  __alarm_manager_call_alarm_delete_sync_fake;

  ret = alarmmgr_remove_alarm(1);
  EXPECT_EQ(ALARMMGR_RESULT_SUCCESS, ret);
}

TEST_F(AlarmLibTest, alarmmgr_remove_all_p)
{
  int ret;

  alarm_manager_call_alarm_delete_all_sync_fake.custom_fake =  __alarm_manager_call_alarm_delete_all_sync_fake;

  ret = alarmmgr_remove_all();
  EXPECT_EQ(ALARMMGR_RESULT_SUCCESS, ret);
}

int enum_fn(alarm_id_t alarm_id, void *user_param)
{
  return 0;
}

TEST_F(AlarmLibTest, alarmmgr_enum_alarm_ids_p)
{
  int ret;

  alarm_manager_call_alarm_get_number_of_ids_sync_fake.custom_fake = __alarm_manager_call_alarm_get_number_of_ids_sync_fake;

  alarm_manager_call_alarm_get_list_of_ids_sync_fake.custom_fake = __alarm_manager_call_alarm_get_list_of_ids_sync_fake;

  ret = alarmmgr_enum_alarm_ids(enum_fn, nullptr);
  EXPECT_EQ(ALARMMGR_RESULT_SUCCESS, ret);
}

TEST_F(AlarmLibTest, alarmmgr_get_next_duetime_p)
{
  int ret;
  time_t next_time;

  alarm_manager_call_alarm_get_next_duetime_sync_fake.custom_fake = __alarm_manager_call_alarm_get_next_duetime_sync_fake;

  ret = alarmmgr_get_next_duetime(1, &next_time);
  EXPECT_EQ(ALARMMGR_RESULT_SUCCESS, ret);
}

int handler(alarm_id_t alarm_id, void *user_param)
{
  return 0;
}

TEST_F(AlarmLibTest, alarmmgr_add_periodic_alarm_withcb_p)
{
  int ret;
  alarm_id_t alarm_id;

  aul_app_get_appid_bypid_fake.custom_fake = __aul_app_get_appid_bypid_fake;
  alarm_manager_call_alarm_create_periodic_sync_fake.custom_fake = __alarm_manager_call_alarm_create_periodic_sync_fake;

  ret = alarmmgr_add_periodic_alarm_withcb(1000, CUT_OFF, handler, NULL,
      &alarm_id);
  EXPECT_EQ(ALARMMGR_RESULT_SUCCESS, ret);
}

TEST_F(AlarmLibTest, alarmmgr_add_reference_periodic_alarm_withcb_p)
{
  int ret;
  alarm_id_t alarm_id;

  aul_app_get_appid_bypid_fake.custom_fake = __aul_app_get_appid_bypid_fake;
  alarm_manager_call_alarm_create_periodic_sync_fake.custom_fake = __alarm_manager_call_alarm_create_periodic_sync_fake;

  ret = alarmmgr_add_reference_periodic_alarm_withcb(1000, handler, NULL,
      &alarm_id);
  EXPECT_EQ(ALARMMGR_RESULT_SUCCESS, ret);
}

TEST_F(AlarmLibTest, alarmmgr_set_systime_p)
{
  int ret;
  time_t current_time;

  time(&current_time);

  alarm_manager_call_alarm_set_time_sync_fake.custom_fake = __alarm_manager_call_alarm_set_time_sync_fake;

  ret = alarmmgr_set_systime(current_time);
  EXPECT_EQ(ALARMMGR_RESULT_SUCCESS, ret);
}

TEST_F(AlarmLibTest, alarmmgr_set_systime_with_propagation_delay_p)
{
  int ret;
  struct timespec newtime;
  struct timespec reqtime;

  clock_gettime(CLOCK_REALTIME, &newtime);
  clock_gettime(CLOCK_REALTIME, &reqtime);

  alarm_manager_call_alarm_set_time_with_propagation_delay_sync_fake.custom_fake = __alarm_manager_call_alarm_set_time_with_propagation_delay_sync_fake;

  ret = alarmmgr_set_systime_with_propagation_delay(newtime, reqtime);
  EXPECT_EQ(ALARMMGR_RESULT_SUCCESS, ret);
}

int set_time_cb(int result, void *user_param)
{
  return 0;
}

TEST_F(AlarmLibTest, alarmmgr_set_systime_async_p)
{
  int ret;
  time_t current_time;

  alarm_manager_call_alarm_set_time_fake.custom_fake = __alarm_manager_call_alarm_set_time_fake;

  time(&current_time);

  ret = alarmmgr_set_systime_async(current_time, set_time_cb, NULL);
  EXPECT_EQ(ALARMMGR_RESULT_SUCCESS, ret);
}

TEST_F(AlarmLibTest, alarmmgr_set_systime_with_propagation_delay_async_p)
{
  int ret;
  struct timespec newtime;
  struct timespec reqtime;

  alarm_manager_call_alarm_set_time_with_propagation_delay_fake.custom_fake = __alarm_manager_call_alarm_set_time_with_propagation_delay_fake;

  clock_gettime(CLOCK_REALTIME, &newtime);
  clock_gettime(CLOCK_REALTIME, &reqtime);

  newtime.tv_sec += 100;
  reqtime.tv_sec += 100;

  ret = alarmmgr_set_systime_with_propagation_delay_async(newtime,
      reqtime, set_time_cb, NULL);
  EXPECT_EQ(ALARMMGR_RESULT_SUCCESS, ret);
}

TEST_F(AlarmLibTest, alarmmgr_set_timezone_p)
{
  int ret;
  char zone[] = "/usr/share/zoneinfo/Asia/Seoul";

  alarm_manager_call_alarm_set_timezone_sync_fake.custom_fake = __alarm_manager_call_alarm_set_timezone_sync_fake;

  ret = alarmmgr_set_timezone(zone);
  EXPECT_EQ(ALARMMGR_RESULT_SUCCESS, ret);
}

TEST_F(AlarmLibTest, alarmmgr_set_globla_p)
{
  int ret;

  alarm_manager_call_alarm_set_global_sync_fake.custom_fake = __alarm_manager_call_alarm_set_global_sync_fake;

  ret = alarmmgr_set_global(1, true);
  EXPECT_EQ(ALARMMGR_RESULT_SUCCESS, ret);
}

TEST_F(AlarmLibTest, alarmmgr_get_globla_p)
{
  int ret;
  bool global;

  alarm_manager_call_alarm_get_global_sync_fake.custom_fake = __alarm_manager_call_alarm_get_global_sync_fake;

  ret = alarmmgr_get_global(1, &global);
  EXPECT_EQ(ALARMMGR_RESULT_SUCCESS, ret);
}

TEST_F(AlarmLibTest, alarmmgr_update_alarm_p)
{
  int ret;
  alarm_date_t test_time;

  alarm_manager_call_alarm_update_sync_fake.custom_fake = __alarm_manager_call_alarm_update_sync_fake;

  test_time.year = 2022;
  test_time.month = 1;
  test_time.day = 1;

  test_time.hour = 1;
  test_time.min = 1;
  test_time.sec = 5;

  ret = alarmmgr_set_time(alarm, test_time);
  EXPECT_EQ(ALARMMGR_RESULT_SUCCESS, ret);

  ret = alarmmgr_update_alarm(1, alarm, ALARM_UPDATE_FLAG_TIME);
  EXPECT_EQ(ALARMMGR_RESULT_SUCCESS, ret);
}
